@@ -2,12 +2,11 @@
 import datetime
 import json
 import re
-from six.moves import urllib
+import urllib.parse
 
 import httmock
 import jwt
 import requests
-import six
 
 from girder.exceptions import ValidationException
 from girder.models.setting import Setting
@@ -34,7 +33,7 @@ def tearDownModule():
 class OauthTest(base.TestCase):
 
     def setUp(self):
-        base.TestCase.setUp(self)
+        super().setUp()
 
         self.adminUser = User().createUser(
             email='rocky@phila.pa.us',
@@ -133,13 +132,13 @@ class OauthTest(base.TestCase):
             callbackLoc = urllib.parse.urlparse(resp.headers['location'])
             callbackLocQuery = urllib.parse.parse_qs(callbackLoc.query)
             callbackParams = {
-                key: val[0] for key, val in six.viewitems(callbackLocQuery)
+                key: val[0] for key, val in callbackLocQuery.items()
             }
             return callbackParams
 
         redirect = 'http://localhost/#foo/bar?token={girderToken}'
 
-        class EventHandler(object):
+        class EventHandler:
             def __init__(self):
                 self.state = ''
 
@@ -206,7 +205,7 @@ class OauthTest(base.TestCase):
             callbackLocQuery = urllib.parse.parse_qs(callbackLoc.query)
             self.assertNotHasKeys(callbackLocQuery, ('error',))
             callbackParams = {
-                key: val[0] for key, val in six.viewitems(callbackLocQuery)
+                key: val[0] for key, val in callbackLocQuery.items()
             }
             return callbackParams
 
@@ -242,7 +241,7 @@ class OauthTest(base.TestCase):
         self.assertIsInstance(resp.json, dict)
         self.assertEqual(len(resp.json), 1)
         self.assertIn(providerInfo['name'], resp.json)
-        six.assertRegex(self, resp.json[providerInfo['name']], providerInfo['url_re'])
+        self.assertRegex(resp.json[providerInfo['name']], providerInfo['url_re'])
 
         # This will need to be called several times, to get fresh tokens
         def getProviderResp():
@@ -254,10 +253,10 @@ class OauthTest(base.TestCase):
             self.assertIsInstance(resp.json, list)
             self.assertEqual(len(resp.json), 1)
             providerResp = resp.json[0]
-            self.assertSetEqual(set(six.viewkeys(providerResp)), {'id', 'name', 'url'})
+            self.assertSetEqual(set(providerResp.keys()), {'id', 'name', 'url'})
             self.assertEqual(providerResp['id'], providerInfo['id'])
             self.assertEqual(providerResp['name'], providerInfo['name'])
-            six.assertRegex(self, providerResp['url'], providerInfo['url_re'])
+            self.assertRegex(providerResp['url'], providerInfo['url_re'])
             redirectParams = urllib.parse.parse_qs(
                 urllib.parse.urlparse(providerResp['url']).query)
             csrfTokenParts = redirectParams['state'][0].partition('.')
@@ -299,7 +298,7 @@ class OauthTest(base.TestCase):
             callbackLocQuery = urllib.parse.parse_qs(callbackLoc.query)
             self.assertNotHasKeys(callbackLocQuery, ('error',))
             callbackParams = {
-                key: val[0] for key, val in six.viewitems(callbackLocQuery)
+                key: val[0] for key, val in callbackLocQuery.items()
             }
             return callbackParams
 
@@ -502,8 +501,8 @@ class OauthTest(base.TestCase):
                     })
                 }
             try:
-                six.assertRegex(
-                    self, params['redirect_uri'][0], providerInfo['allowed_callback_re'])
+                self.assertRegex(
+                    params['redirect_uri'][0], providerInfo['allowed_callback_re'])
                 state = params['state'][0]
                 # Nothing to test for state, since provider doesn't care
             except (KeyError, AssertionError) as e:
@@ -540,9 +539,9 @@ class OauthTest(base.TestCase):
             try:
                 self.assertEqual(params['grant_type'], ['authorization_code'])
                 self.assertEqual(params['client_secret'], [providerInfo['client_secret']['value']])
-                six.assertRegex(
-                    self, params['redirect_uri'][0], providerInfo['allowed_callback_re'])
-                for account in six.viewvalues(providerInfo['accounts']):
+                self.assertRegex(
+                    params['redirect_uri'][0], providerInfo['allowed_callback_re'])
+                for account in providerInfo['accounts'].values():
                     if account['auth_code'] == params['code'][0]:
                         break
                 else:
@@ -574,7 +573,7 @@ class OauthTest(base.TestCase):
         @httmock.urlmatch(scheme='https', netloc=r'^openidconnect\.googleapis\.com$',
                           path=r'^/v1/userinfo$', method='GET')
         def mockGoogleApi(url, request):
-            for account in six.viewvalues(providerInfo['accounts']):
+            for account in providerInfo['accounts'].values():
                 if 'Bearer %s' % account['access_token'] == request.headers['Authorization']:
                     break
             else:
@@ -664,7 +663,7 @@ class OauthTest(base.TestCase):
                     })
                 }
             try:
-                six.assertRegex(self, redirectUri, providerInfo['allowed_callback_re'])
+                self.assertRegex(redirectUri, providerInfo['allowed_callback_re'])
                 state = params['state'][0]
                 # Nothing to test for state, since provider doesn't care
                 self.assertEqual(params['scope'], ['user:email'])
@@ -699,14 +698,14 @@ class OauthTest(base.TestCase):
                     })
                 }
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if account['auth_code'] == params['code'][0]:
                         break
                 else:
                     self.fail()
                 self.assertEqual(params['client_secret'], [providerInfo['client_secret']['value']])
-                six.assertRegex(
-                    self, params['redirect_uri'][0], providerInfo['allowed_callback_re'])
+                self.assertRegex(
+                    params['redirect_uri'][0], providerInfo['allowed_callback_re'])
             except (KeyError, AssertionError) as e:
                 returnBody = json.dumps({
                     'error': repr(e),
@@ -729,7 +728,7 @@ class OauthTest(base.TestCase):
         @httmock.urlmatch(scheme='https', netloc='^api.github.com$', path='^/user$', method='GET')
         def mockGithubApiUser(url, request):
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if 'token %s' % account['access_token'] == request.headers['Authorization']:
                         break
                 else:
@@ -751,7 +750,7 @@ class OauthTest(base.TestCase):
                           path='^/user/emails$', method='GET')
         def mockGithubApiEmail(url, request):
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if 'token %s' % account['access_token'] == request.headers['Authorization']:
                         break
                 else:
@@ -788,7 +787,7 @@ class OauthTest(base.TestCase):
         @httmock.urlmatch(scheme='https', netloc='^api.github.com$', path='^/user$', method='GET')
         def mockGithubUserWithoutName(url, request):
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if 'token %s' % account['access_token'] == request.headers['Authorization']:
                         break
                 else:
@@ -900,8 +899,8 @@ class OauthTest(base.TestCase):
                     })
                 }
             try:
-                six.assertRegex(
-                    self, params['redirect_uri'][0], providerInfo['allowed_callback_re'])
+                self.assertRegex(
+                    params['redirect_uri'][0], providerInfo['allowed_callback_re'])
                 state = params['state'][0]
                 # Nothing to test for state, since provider doesn't care
             except (KeyError, AssertionError) as e:
@@ -926,7 +925,7 @@ class OauthTest(base.TestCase):
                           path='^/v2/oauth2/userinfo$', method='GET')
         def mockGlobusUserInfo(url, request):
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if 'Bearer %s' % account['access_token'] == \
                             request.headers['Authorization']:
                         break
@@ -962,14 +961,14 @@ class OauthTest(base.TestCase):
                     })
                 }
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if account['auth_code'] == params['code'][0]:
                         break
                 else:
                     self.fail()
                 self.assertEqual(params['client_secret'], [providerInfo['client_secret']['value']])
-                six.assertRegex(
-                    self, params['redirect_uri'][0], providerInfo['allowed_callback_re'])
+                self.assertRegex(
+                    params['redirect_uri'][0], providerInfo['allowed_callback_re'])
             except (KeyError, AssertionError) as e:
                 returnBody = json.dumps({
                     'error': repr(e),
@@ -1061,8 +1060,8 @@ class OauthTest(base.TestCase):
             try:
                 params = urllib.parse.parse_qs(url.query)
                 self.assertEqual(params['client_id'], [providerInfo['client_id']['value']])
-                six.assertRegex(
-                    self, params['redirect_uri'][0], providerInfo['allowed_callback_re'])
+                self.assertRegex(
+                    params['redirect_uri'][0], providerInfo['allowed_callback_re'])
             except (KeyError, AssertionError) as e:
                 return {
                     'status_code': 200,
@@ -1102,13 +1101,13 @@ class OauthTest(base.TestCase):
                 params = urllib.parse.parse_qs(request.body)
                 self.assertEqual(params['grant_type'], ['authorization_code'])
                 self.assertEqual(params['client_id'], [providerInfo['client_id']['value']])
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if account['auth_code'] == params['code'][0]:
                         break
                 else:
                     self.fail()
-                six.assertRegex(
-                    self, params['redirect_uri'][0], providerInfo['allowed_callback_re'])
+                self.assertRegex(
+                    params['redirect_uri'][0], providerInfo['allowed_callback_re'])
             except (KeyError, AssertionError) as e:
                 return {
                     'status_code': 400,
@@ -1136,7 +1135,7 @@ class OauthTest(base.TestCase):
                           path=r'^/v1/people/~(?::\(.+\)?)$', method='GET')
         def mockLinkedinApi(url, request):
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if 'Bearer %s' % account['access_token'] == request.headers['Authorization']:
                         break
                 else:
@@ -1249,7 +1248,7 @@ class OauthTest(base.TestCase):
                     })
                 }
             try:
-                six.assertRegex(self, redirectUri, providerInfo['allowed_callback_re'])
+                self.assertRegex(redirectUri, providerInfo['allowed_callback_re'])
                 state = params['state'][0]
                 # Nothing to test for state, since provider doesn't care
                 self.assertEqual(params['scope'], ['account'])
@@ -1286,14 +1285,14 @@ class OauthTest(base.TestCase):
                     })
                 }
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if account['auth_code'] == params['code'][0]:
                         break
                 else:
                     self.fail()
                 self.assertEqual(params['client_secret'], [providerInfo['client_secret']['value']])
-                six.assertRegex(
-                    self, params['redirect_uri'][0], providerInfo['allowed_callback_re'])
+                self.assertRegex(
+                    params['redirect_uri'][0], providerInfo['allowed_callback_re'])
             except (KeyError, AssertionError) as e:
                 returnBody = json.dumps({
                     'error': repr(e),
@@ -1317,7 +1316,7 @@ class OauthTest(base.TestCase):
                           path='^/2.0/user$', method='GET')
         def mockBitbucketApiUser(url, request):
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if 'Bearer %s' % account['access_token'] == request.headers['Authorization']:
                         break
                 else:
@@ -1344,7 +1343,7 @@ class OauthTest(base.TestCase):
                           path='^/2.0/user/emails$', method='GET')
         def mockBitbucketApiEmail(url, request):
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if 'Bearer %s' % account['access_token'] == request.headers['Authorization']:
                         break
                 else:
@@ -1379,6 +1378,107 @@ class OauthTest(base.TestCase):
         ):
             self._testOauth(providerInfo)
 
+    def XtestMicrosoftOauth(self):  # noqa
+        # This test is disabled for now:
+        # https://github.com/girder/girder/pull/3393#issuecomment-1113589758
+        providerInfo = {
+            'id': 'microsoft',
+            'name': 'Microsoft',
+            'client_id': {
+                'key': PluginSettings.MICROSOFT_CLIENT_ID,
+                'value': 'microsoft_test_client_id'
+            },
+            'client_secret': {
+                'key': PluginSettings.MICROSOFT_CLIENT_SECRET,
+                'value': 'microsoft_test_client_secret'
+            },
+            'allowed_callback_re':
+                r'^http://127\.0\.0\.1(?::\d+)?'
+                r'/api/v1/oauth/microsoft/callback$',
+            'url_re': r'^https://login\.microsoftonline\.com/organizations/oauth2/v2\.0/authorize',
+            'accounts': {
+                'existing': {
+                    'auth_code': 'microsoft_existing_auth_code',
+                    'access_token': 'microsoft_existing_test_token',
+                    'user': {
+                        'login': self.adminUser['login'],
+                        'email': self.adminUser['email'],
+                        'firstName': self.adminUser['firstName'],
+                        'lastName': self.adminUser['lastName'],
+                        'oauth': {
+                            'provider': 'microsoft',
+                            'id': '2399'
+                        }
+                    }
+                },
+                'new': {
+                    'auth_code': 'microsoft_new_auth_code',
+                    'access_token': 'microsoft_new_test_token',
+                    'user': {
+                        # login may be provided externally by Microsoft; for
+                        # simplicity here, do not use a username with whitespace
+                        # or underscores
+                        'login': 'drago',
+                        'email': 'metaphor@labs.ussr.gov',
+                        'firstName': 'Ivan',
+                        'lastName': 'Drago',
+                        'oauth': {
+                            'provider': 'microsoft',
+                            'id': 1985,
+                        }
+                    }
+                }
+            }
+        }
+
+        @httmock.urlmatch(scheme='https', netloc='^login.microsoftonline.com$',
+                          path='^/organizations/oauth2/v2.0/authorize$', method='GET')
+        def mockMicrosoftRedirect(url, request):
+            redirectUri = None
+            try:
+                params = urllib.parse.parse_qs(url.query)
+                # Check redirect_uri first, so other errors can still redirect
+                redirectUri = params['redirect_uri'][0]
+                self.assertEqual(params['client_id'], [providerInfo['client_id']['value']])
+            except (KeyError, AssertionError) as e:
+                return {
+                    'status_code': 404,
+                    'content': json.dumps({
+                        'error': repr(e)
+                    })
+                }
+            try:
+                self.assertRegex(redirectUri, providerInfo['allowed_callback_re'])
+                state = params['state'][0]
+                # Nothing to test for state, since provider doesn't care
+                self.assertEqual(params['scope'], ['account'])
+            except (KeyError, AssertionError) as e:
+                returnQuery = urllib.parse.urlencode({
+                    'error': repr(e),
+                    'error_description': repr(e)
+                })
+            else:
+                returnQuery = urllib.parse.urlencode({
+                    'state': state,
+                    'code': providerInfo['accounts'][self.accountType]['auth_code']
+                })
+            return {
+                'status_code': 302,
+                'headers': {
+                    'Location': '%s?%s' % (redirectUri, returnQuery)
+                }
+            }
+
+        with httmock.HTTMock(
+            mockMicrosoftRedirect,
+            # mockMicrosoftToken,
+            # mockMicrosoftApiUser,
+            # mockMicrosoftApiEmail,
+            # Must keep 'mockOtherRequest' last
+            self.mockOtherRequest,
+        ):
+            self._testOauth(providerInfo)
+
     def testBoxOauth(self):  # noqa
         providerInfo = {
             'id': 'box',
@@ -1445,7 +1545,7 @@ class OauthTest(base.TestCase):
                     })
                 }
             try:
-                six.assertRegex(self, redirectUri, providerInfo['allowed_callback_re'])
+                self.assertRegex(redirectUri, providerInfo['allowed_callback_re'])
                 state = params['state'][0]
                 # Nothing to test for state, since provider doesn't care
             except (KeyError, AssertionError) as e:
@@ -1479,7 +1579,7 @@ class OauthTest(base.TestCase):
                     })
                 }
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if account['auth_code'] == params['code'][0]:
                         break
                 else:
@@ -1508,7 +1608,7 @@ class OauthTest(base.TestCase):
                           path='^/2.0/users/me$', method='GET')
         def mockBoxApiUser(url, request):
             try:
-                for account in six.viewvalues(providerInfo['accounts']):
+                for account in providerInfo['accounts'].values():
                     if 'Bearer %s' % account['access_token'] == request.headers['Authorization']:
                         break
                 else:
